/*
 * Screen Manager class 
 * Created on 19 aot 2009 ( Time 12:19:38 )
 */

package org.telosys.samples.screen.author ;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenRequestParameters;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.core.StandardScreenManager;
import org.objectweb.telosys.screen.env.ScreenSession;

import org.telosys.samples.vo.bean.Author;
import org.telosys.samples.vo.dao.AuthorDAO;


/**
 * Screen Manager for entity Author ( server-side actions implementation )
 *
 * @author xx
 *
 */
public class AuthorMgr extends StandardScreenManager
{
    /**
     * Default constructor.
     */
    public AuthorMgr()
    {
		super( AuthorData.class ); // Use the default database
		// super( AuthorData.class, 1 ); // Use the database 1
	}

    
	@Override
	public void onCreate(ScreenContext screenContext,
			ScreenRequestParameters params, ScreenSession screenSession,
			DatabaseSession dbSession) throws TelosysException {
		super.onCreate(screenContext, params, screenSession, dbSession);
		System.out.println("onCreate");
	}


	@Override
	public void onReuse(ScreenContext screenContext,
			ScreenRequestParameters params, ScreenSession screenSession,
			DatabaseSession dbSession) throws TelosysException {
		super.onReuse(screenContext, params, screenSession, dbSession);
		System.out.println("onReuse");
	}


	/**
	 * Set the Primary Key in the bean's attributes 
	 * @param bean
	 * @param request
	 * @throws TelosysException
	 */
	private void setPrimaryKey(Author bean, ScreenRequest request) throws TelosysException 
	{
		bean.setId( request.getParamAsInt("id") ); // int : AUTHOR.ID
	}

	/**
	 * Returns true if the request contains parameters for the Primary Key
	 * @param request
	 * @return
	 */
	private boolean hasPrimaryKey(ScreenRequest request) 
	{
		if ( request.getParameter("id") == null ) return false ;
		return true ;
	}
    
	/**
	 * Returns the DAO for the "Author" entity
	 * @param dbSession
	 * @return
	 * @throws TelosysException
	 */
	private AuthorDAO getAuthorDAO(DatabaseSession dbSession) throws TelosysException
	{
    	return (AuthorDAO) getDAO( Author.class, dbSession ) ;
	}
	
	/* 
	 * Implementation of the standard "LOAD" action
	 */
	public void load(ScreenData screenData, ScreenContext context, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException 
	{
		System.out.println("AuthorMgr.load()...");
		AuthorData data = (AuthorData) screenData ;
		Author bean = new Author();
		setPrimaryKey(bean, request); 
		AuthorDAO dao = getAuthorDAO(dbSession) ;

		int iRet = dao.load(bean, dbSession);
		if ( iRet > 0 ) // if found 
		{
			System.out.println("AuthorMgr.load() : found ");
			//--- Populate the Screen Data ...
			data.setAuthor(bean);
			data.setFound(); // Consider as "found"
			//--- Load other related data if necessary
			// use other VO and DAO
		}
		else
		{
			System.out.println("AuthorMgr.load() : not found ");
			data.clearElements();
			data.setNotFound(); // Consider as "not found"
		}
	}
	
	/* 
	 * Implementation of the standard "SAVE" action
	 */
	public void save(ScreenData screenData, ScreenContext context, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException 
	{
		AuthorData data = (AuthorData) screenData ;
    	//--- SAVE BEAN ( after set )
		Author bean = data.getAuthor();
    	if ( bean != null )
    	{
			AuthorDAO dao = getAuthorDAO(dbSession) ;
    		dao.save(bean, dbSession);
    	}
	}

	/* 
	 * Implementation of the standard "INSERT" action
	 */
	public void insert(ScreenData screenData, ScreenContext context, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException 
	{
		throw new TelosysException("INSERT action not implemented");
	}

	/* 
	 * Implementation of the standard "UPDATE" action
	 */
	public void update(ScreenData screenData, ScreenContext context, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException 
	{
		throw new TelosysException("UPDATE action not implemented");
	}

	/* 
	 * Implementation of the standard "DELETE" action
	 */
	public void delete(ScreenData screenData, ScreenContext context, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException 
	{
		if ( hasPrimaryKey(request) )
		{
			delete_by_primary_key(request, response, dbSession);
		}
		else
		{
			delete_current(context, request, response, dbSession);
		}
		context.clearCurrentData();
	}
	
	/**
	 * Deletes the entity using the Primary Key received as request parameter(s)
	 * @param request
	 * @param response
	 * @param dbSession
	 * @throws TelosysException
	 */
	private void delete_by_primary_key(ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException 
	{
    	//--- DELETE the PRIMARY KEY RECEIVED as REQUEST PARAMETER
    	Author bean = new Author();
		setPrimaryKey(bean, request); 
		AuthorDAO dao = getAuthorDAO(dbSession) ;

		int iRet = dao.delete(bean, dbSession);
		if ( iRet > 0 )
		{
    		response.setReturnMessage("Bean deleted by primary key");
		}
		else
		{
    		response.setReturnMessage("No bean deleted for the given primary key");
		}
	}

	/**
	 * Deletes the entity using the Primary Key of the element stored in the current ScreenData
	 * @param context
	 * @param request
	 * @param response
	 * @param dbSession
	 * @throws TelosysException
	 */
	private void delete_current(ScreenContext context, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException 
	{
    	//--- DELETE CURRENT BEAN    	
		AuthorData data = (AuthorData) context.getCurrentData() ;
		Author bean = data.getAuthor();
    	
    	if ( bean != null )
    	{
			AuthorDAO dao = getAuthorDAO(dbSession) ;
    		dao.delete(bean, dbSession);
    		data.clearElements();
    		response.setReturnMessage("Current bean has been deleted");
    	}
    	else
    	{
    		response.setReturnMessage("No current bean, nothing deleted");
    	}
	}
}
